#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Simulation
- Name     : Geomorphology
- ID       : sim_geomorphology

Description
----------
This library contains tools for the simulation of geomorphological processes.
'''

from PySAGA.helper import Tool_Wrapper

def Gravitational_Process_Path_Model(
    DEM=None,
    RELEASE_AREAS=None,
    MATERIAL=None,
    FRICTION_ANGLE_GRID=None,
    SLOPE_IMPACT_GRID=None,
    FRICTION_MU_GRID=None,
    FRICTION_MASS_TO_DRAG_GRID=None,
    OBJECTS=None,
    PROCESS_AREA=None,
    DEPOSITION=None,
    MAX_VELOCITY=None,
    STOP_POSITIONS=None,
    HAZARD_PATHS=None,
    HAZARD_SOURCES=None,
    HAZARD_SOURCES_MATERIAL=None,
    MATERIAL_FLUX=None,
    PROCESS_PATH_MODEL=None,
    RW_SLOPE_THRES=None,
    RW_EXPONENT=None,
    RW_PERSISTENCE=None,
    GPP_ITERATIONS=None,
    GPP_PROCESSING_ORDER=None,
    GPP_SEED=None,
    FRICTION_MODEL=None,
    FRICTION_THRES_FREE_FALL=None,
    FRICTION_METHOD_IMPACT=None,
    FRICTION_IMPACT_REDUCTION=None,
    FRICTION_ANGLE=None,
    FRICTION_MU=None,
    FRICTION_MODE_OF_MOTION=None,
    FRICTION_MASS_TO_DRAG=None,
    FRICTION_INIT_VELOCITY=None,
    DEPOSITION_MODEL=None,
    DEPOSITION_INITIAL=None,
    DEPOSITION_SLOPE_THRES=None,
    DEPOSITION_VELOCITY_THRES=None,
    DEPOSITION_MAX=None,
    DEPOSITION_MIN_PATH=None,
    SINK_MIN_SLOPE=None, Verbose=2):
    '''
    Gravitational Process Path Model
    ----------
    [sim_geomorphology.0]\n
    The Gravitational Process Path (GPP) model can be used to simulate the process path and run-out area of gravitational processes based on a digital terrain model (DTM). The conceptual model combines several components (process path, run-out length, sink filling and material deposition) to simulate the movement of a mass point from an initiation site to the deposition area. For each component several modeling approaches are provided, which makes the tool configurable for different processes such as rockfall, debris flows or snow avalanches.[[The tool can be applied to regional-scale studies such as natural hazard susceptibility mapping but also contains components for scenario-based modeling of single events. Both the modeling approaches and precursor implementations of the tool have proven their applicability in numerous studies, also including geomorphological research questions such as the delineation of sediment cascades or the study of process connectivity.[[Please provide the reference cited below in your work if you are using the GPP model.[[[Addendum:][The article is not clear about the way the impact on the slope is exactly modelled when the 'Shadow Angle' or '1-parameter' friction model is used. Besides the 'Threshold Angle Free Fall' criterion to determine the location of the first impact, it is also assumed that the particle must leave its own release area (given by its ID) in order to impact. This is actually a conceptual design, taking into account that free fall usually occurs in steep rock faces (release areas), and the fact, that such rockfaces are not characterised very well in a 2.5D elevation model. You can work around that conceptual design by providing a grid describing the 'slope impact areas' as input. Using such a grid disables the 'Threshold Angle Free Fall' parameter.[[[New in version 1.1:][Since version 1.1 the model supports the monitoring of potentially endangered objects like infrastructure and reports from which process paths and release areas objects might be hit. In order to enable this backtracking, the user must provide an 'Objects' grid as input. The grid can be used to store different types or classes of objects, using one-hot categorical data encoding for each object class, i.e. powers of ten: 1, 10, 100, 1000, etc. (all other cells NoData). The 'Hazard Paths' and 'Hazard Sources' output grid will store combinations of these numbers if several different classes were hit from a grid cell, allowing to analyse which object classes might be hit from which location.[[[New in version 1.2:][Since version 1.2 the model supports the optional output of a grid with the material flux. This requires a 'Material' grid as input. The grid shows the height of the material that has passed in total through each grid cell.[[[Version 1.3:][Version 1.3 includes a fix for material deposition along the process path (amounts and the update of available material for subsequent runs) and improves the output of material flux.[[[Version 1.4:][Since version 1.4 two separate 'Endangered Objects' output grids are created, one encoding the process path cells, the other only the source cells from which objects have been hit. These optional output parameters also have been renamed ('HAZARD_PATHS', 'HAZARD_SOURCES') to improve the legibility of the parameter interface. A third grid, showing the total amount of material in each source cell that has hit objects from that position, has also been added. This output requires a material grid as input in order to calculate material flux. To determine the total amount, the highest material flux observed to hit an object per path and iteration is summed up and converted back to a material height per cell (comparable to the amount specified in the material input grid.)\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM. Digital elevation model [m].
    - RELEASE_AREAS [`input grid`] : Release Areas. Release areas encoded by unique integer IDs, all other cells NoData [-].
    - MATERIAL [`optional input grid`] : Material. Height of material available in each start cell [m].
    - FRICTION_ANGLE_GRID [`optional input grid`] : Friction Angle. Spatially distributed friction angles [degree]. Optionally used with the Geometric Gradient, Fahrboeschung's angle or Shadow Angle friction model.
    - SLOPE_IMPACT_GRID [`optional input grid`] : Slope Impact Areas. Slope impact grid, impact areas encoded with valid values, all other NoData. Optionally used with the Shadow Angle or the 1-parameter friction model.
    - FRICTION_MU_GRID [`optional input grid`] : Friction Parameter Mu. Spatially distributed friction parameter mu [-], optionally used with the 1-parameter friction model or the PCM Model.
    - FRICTION_MASS_TO_DRAG_GRID [`optional input grid`] : Mass to Drag Ratio. Spatially distributed mass to drag ratio [m], optionally used with the PCM Model.
    - OBJECTS [`optional input grid`] : Objects. Potentially endangered objects (like infrastructure) to monitor, using one-hot categorical data encoding for each object class [1, 10, 100, 1000, ...].
    - PROCESS_AREA [`output grid`] : Process Area. Delineated process area with encoded transition frequencies [count].
    - DEPOSITION [`output grid`] : Deposition. Height of the material deposited in each cell [m]. Optional output in case a grid with material amounts is provided as input.
    - MAX_VELOCITY [`output grid`] : Maximum Velocity. Maximum velocity observed in each cell [m/s]. Optional output of the 1-parameter friction model and the PCM Model.
    - STOP_POSITIONS [`output grid`] : Stopping Positions. Stopping positions, showing cells in which the run-out length has been reached [count].
    - HAZARD_PATHS [`output grid`] : Hazard Paths. Process path cells from which objects were hit. Cell values indicate which object classes were hit [combination of object classes]. Optional output in case a grid with potentially endangered objects is provided as input.
    - HAZARD_SOURCES [`output grid`] : Hazard Sources. Source (release area) cells from which objects were hit. Cell values indicate which object classes were hit [combination of object classes]. Optional output in case a grid with potentially endangered objects is provided as input.
    - HAZARD_SOURCES_MATERIAL [`output grid`] : Hazard Sources Material. Source (release area) cells from which objects were hit. Cell values indicate the material amount [m/cell] that has hit objects from that source cell. Optional output in case grids with material amounts and potentially endangered objects are provided as input.
    - MATERIAL_FLUX [`output grid`] : Material Flux. Amount of material that has passed through each cell [m]. Optional output in case a grid with material amounts is provided as input.
    - PROCESS_PATH_MODEL [`choice`] : Model. Available Choices: [0] Maximum Slope [1] Random Walk Default: 1 Choose a process path model.
    - RW_SLOPE_THRES [`floating point number`] : Slope Threshold. Minimum: 0.001000 Maximum: 90.000000 Default: 40.000000 In case the local slope is greater as this threshold [degree], no lateral spreading is modeled.
    - RW_EXPONENT [`floating point number`] : Exponent. Minimum: 1.000000 Default: 2.000000 The exponent [-] is controlling the amount of lateral spreading in case the local slope is in between zero and the slope threshold.
    - RW_PERSISTENCE [`floating point number`] : Persistence Factor. Minimum: 1.000000 Default: 1.500000 Factor [-] used as weight for the current flow direction. A higher factor reduces abrupt changes in flow direction.
    - GPP_ITERATIONS [`integer number`] : Iterations. Minimum: 1 Default: 1000 The number of model runs from each start cell [-].
    - GPP_PROCESSING_ORDER [`choice`] : Processing Order. Available Choices: [0] RAs in Sequence [1] RAs in Sequence per Iteration [2] RAs in Parallel per Iteration Default: 2 Choose the processing order.
    - GPP_SEED [`integer number`] : Seed Value. Minimum: 1 Default: 1 The seed value used to initialize the pseudo-random number generator. A value of 1 will initialize the generator with the current time, higher numbers will always produce the same succession of values for each seed value [-].
    - FRICTION_MODEL [`choice`] : Model. Available Choices: [0] None [1] Geometric Gradient (Heim 1932) [2] Fahrboeschung Principle (Heim 1932) [3] Shadow Angle (Evans & Hungr 1988) [4] 1-parameter friction model (Scheidegger 1975) [5] PCM Model (Perla et al. 1980) Default: 0 Choose a friction model.
    - FRICTION_THRES_FREE_FALL [`floating point number`] : Threshold Angle Free Fall. Minimum: 0.000000 Default: 60.000000 The minimum slope angle [degree] between start cell and current cell for modeling free fall with the Shadow Angle or the 1-parameter friction model.
    - FRICTION_METHOD_IMPACT [`choice`] : Method Impact. Available Choices: [0] Energy Reduction (Scheidegger 1975) [1] Preserved Component of Velocity (Kirkby & Statham 1975) Default: 0 Choose the velocity calculation on slope impact with the 1-parameter friction model.
    - FRICTION_IMPACT_REDUCTION [`floating point number`] : Reduction. Minimum: 0.000000 Maximum: 100.000000 Default: 75.000000 The energy reduction [%] on slope impact with the 1-parameter friction model.
    - FRICTION_ANGLE [`floating point number`] : Friction Angle. Minimum: 0.000000 Maximum: 90.000000 Default: 30.000000 Friction angle [degree] used as Geometric Gradient, Fahrboeschung's angle or Shadow Angle.
    - FRICTION_MU [`floating point number`] : Mu. Minimum: 0.000000 Default: 0.250000 The (constant) friction parameter mu [-] used with the 1-parameter friction model or the PCM Model.
    - FRICTION_MODE_OF_MOTION [`choice`] : Mode of Motion. Available Choices: [0] Sliding [1] Rolling Default: 0 Choose the mode of motion on hillslope with the 1-parameter friction model.
    - FRICTION_MASS_TO_DRAG [`floating point number`] : Mass to Drag Ratio. Minimum: 0.000000 Default: 200.000000 The (constant) mass to drag ratio [m] used with the PCM Model.
    - FRICTION_INIT_VELOCITY [`floating point number`] : Initial Velocity. Minimum: 0.000000 Default: 1.000000 The initial velocity [m/s] used with the PCM Model.
    - DEPOSITION_MODEL [`choice`] : Model. Available Choices: [0] None [1] On Stop [2] Slope & On Stop [3] Velocity & On Stop [4] min(Slope,Velocity) & On Stop Default: 0 Choose a deposition model.
    - DEPOSITION_INITIAL [`floating point number`] : Initial Deposition on Stop. Minimum: 0.000000 Maximum: 100.000000 Default: 20.000000 The percentage of available material (per run) initially deposited at the stopping position [%].
    - DEPOSITION_SLOPE_THRES [`floating point number`] : Slope Threshold. Minimum: 0.000000 Maximum: 90.000000 Default: 20.000000 The slope angle below which the deposition of material is starting [degree].
    - DEPOSITION_VELOCITY_THRES [`floating point number`] : Velocity Threshold. Minimum: 0.000000 Default: 15.000000 The velocity below which the deposition of material is starting [m/s].
    - DEPOSITION_MAX [`floating point number`] : Maximum Deposition along Path. Minimum: 0.000000 Maximum: 100.000000 Default: 20.000000 The percentage of available material (per run) which is deposited at most (slope or velocity equal zero) [%].
    - DEPOSITION_MIN_PATH [`floating point number`] : Minimum Path Length. Minimum: 0.000000 Default: 100.000000 The minimum path length which has to be reached before material deposition is enabled [m].
    - SINK_MIN_SLOPE [`floating point number`] : Minimum Slope. Minimum: 0.000000 Maximum: 90.000000 Default: 2.500000 The minimum slope to preserve on sink filling [degree].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_geomorphology', '0', 'Gravitational Process Path Model')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('RELEASE_AREAS', RELEASE_AREAS)
        Tool.Set_Input ('MATERIAL', MATERIAL)
        Tool.Set_Input ('FRICTION_ANGLE_GRID', FRICTION_ANGLE_GRID)
        Tool.Set_Input ('SLOPE_IMPACT_GRID', SLOPE_IMPACT_GRID)
        Tool.Set_Input ('FRICTION_MU_GRID', FRICTION_MU_GRID)
        Tool.Set_Input ('FRICTION_MASS_TO_DRAG_GRID', FRICTION_MASS_TO_DRAG_GRID)
        Tool.Set_Input ('OBJECTS', OBJECTS)
        Tool.Set_Output('PROCESS_AREA', PROCESS_AREA)
        Tool.Set_Output('DEPOSITION', DEPOSITION)
        Tool.Set_Output('MAX_VELOCITY', MAX_VELOCITY)
        Tool.Set_Output('STOP_POSITIONS', STOP_POSITIONS)
        Tool.Set_Output('HAZARD_PATHS', HAZARD_PATHS)
        Tool.Set_Output('HAZARD_SOURCES', HAZARD_SOURCES)
        Tool.Set_Output('HAZARD_SOURCES_MATERIAL', HAZARD_SOURCES_MATERIAL)
        Tool.Set_Output('MATERIAL_FLUX', MATERIAL_FLUX)
        Tool.Set_Option('PROCESS_PATH_MODEL', PROCESS_PATH_MODEL)
        Tool.Set_Option('RW_SLOPE_THRES', RW_SLOPE_THRES)
        Tool.Set_Option('RW_EXPONENT', RW_EXPONENT)
        Tool.Set_Option('RW_PERSISTENCE', RW_PERSISTENCE)
        Tool.Set_Option('GPP_ITERATIONS', GPP_ITERATIONS)
        Tool.Set_Option('GPP_PROCESSING_ORDER', GPP_PROCESSING_ORDER)
        Tool.Set_Option('GPP_SEED', GPP_SEED)
        Tool.Set_Option('FRICTION_MODEL', FRICTION_MODEL)
        Tool.Set_Option('FRICTION_THRES_FREE_FALL', FRICTION_THRES_FREE_FALL)
        Tool.Set_Option('FRICTION_METHOD_IMPACT', FRICTION_METHOD_IMPACT)
        Tool.Set_Option('FRICTION_IMPACT_REDUCTION', FRICTION_IMPACT_REDUCTION)
        Tool.Set_Option('FRICTION_ANGLE', FRICTION_ANGLE)
        Tool.Set_Option('FRICTION_MU', FRICTION_MU)
        Tool.Set_Option('FRICTION_MODE_OF_MOTION', FRICTION_MODE_OF_MOTION)
        Tool.Set_Option('FRICTION_MASS_TO_DRAG', FRICTION_MASS_TO_DRAG)
        Tool.Set_Option('FRICTION_INIT_VELOCITY', FRICTION_INIT_VELOCITY)
        Tool.Set_Option('DEPOSITION_MODEL', DEPOSITION_MODEL)
        Tool.Set_Option('DEPOSITION_INITIAL', DEPOSITION_INITIAL)
        Tool.Set_Option('DEPOSITION_SLOPE_THRES', DEPOSITION_SLOPE_THRES)
        Tool.Set_Option('DEPOSITION_VELOCITY_THRES', DEPOSITION_VELOCITY_THRES)
        Tool.Set_Option('DEPOSITION_MAX', DEPOSITION_MAX)
        Tool.Set_Option('DEPOSITION_MIN_PATH', DEPOSITION_MIN_PATH)
        Tool.Set_Option('SINK_MIN_SLOPE', SINK_MIN_SLOPE)
        return Tool.Execute(Verbose)
    return False

def run_tool_sim_geomorphology_0(
    DEM=None,
    RELEASE_AREAS=None,
    MATERIAL=None,
    FRICTION_ANGLE_GRID=None,
    SLOPE_IMPACT_GRID=None,
    FRICTION_MU_GRID=None,
    FRICTION_MASS_TO_DRAG_GRID=None,
    OBJECTS=None,
    PROCESS_AREA=None,
    DEPOSITION=None,
    MAX_VELOCITY=None,
    STOP_POSITIONS=None,
    HAZARD_PATHS=None,
    HAZARD_SOURCES=None,
    HAZARD_SOURCES_MATERIAL=None,
    MATERIAL_FLUX=None,
    PROCESS_PATH_MODEL=None,
    RW_SLOPE_THRES=None,
    RW_EXPONENT=None,
    RW_PERSISTENCE=None,
    GPP_ITERATIONS=None,
    GPP_PROCESSING_ORDER=None,
    GPP_SEED=None,
    FRICTION_MODEL=None,
    FRICTION_THRES_FREE_FALL=None,
    FRICTION_METHOD_IMPACT=None,
    FRICTION_IMPACT_REDUCTION=None,
    FRICTION_ANGLE=None,
    FRICTION_MU=None,
    FRICTION_MODE_OF_MOTION=None,
    FRICTION_MASS_TO_DRAG=None,
    FRICTION_INIT_VELOCITY=None,
    DEPOSITION_MODEL=None,
    DEPOSITION_INITIAL=None,
    DEPOSITION_SLOPE_THRES=None,
    DEPOSITION_VELOCITY_THRES=None,
    DEPOSITION_MAX=None,
    DEPOSITION_MIN_PATH=None,
    SINK_MIN_SLOPE=None, Verbose=2):
    '''
    Gravitational Process Path Model
    ----------
    [sim_geomorphology.0]\n
    The Gravitational Process Path (GPP) model can be used to simulate the process path and run-out area of gravitational processes based on a digital terrain model (DTM). The conceptual model combines several components (process path, run-out length, sink filling and material deposition) to simulate the movement of a mass point from an initiation site to the deposition area. For each component several modeling approaches are provided, which makes the tool configurable for different processes such as rockfall, debris flows or snow avalanches.[[The tool can be applied to regional-scale studies such as natural hazard susceptibility mapping but also contains components for scenario-based modeling of single events. Both the modeling approaches and precursor implementations of the tool have proven their applicability in numerous studies, also including geomorphological research questions such as the delineation of sediment cascades or the study of process connectivity.[[Please provide the reference cited below in your work if you are using the GPP model.[[[Addendum:][The article is not clear about the way the impact on the slope is exactly modelled when the 'Shadow Angle' or '1-parameter' friction model is used. Besides the 'Threshold Angle Free Fall' criterion to determine the location of the first impact, it is also assumed that the particle must leave its own release area (given by its ID) in order to impact. This is actually a conceptual design, taking into account that free fall usually occurs in steep rock faces (release areas), and the fact, that such rockfaces are not characterised very well in a 2.5D elevation model. You can work around that conceptual design by providing a grid describing the 'slope impact areas' as input. Using such a grid disables the 'Threshold Angle Free Fall' parameter.[[[New in version 1.1:][Since version 1.1 the model supports the monitoring of potentially endangered objects like infrastructure and reports from which process paths and release areas objects might be hit. In order to enable this backtracking, the user must provide an 'Objects' grid as input. The grid can be used to store different types or classes of objects, using one-hot categorical data encoding for each object class, i.e. powers of ten: 1, 10, 100, 1000, etc. (all other cells NoData). The 'Hazard Paths' and 'Hazard Sources' output grid will store combinations of these numbers if several different classes were hit from a grid cell, allowing to analyse which object classes might be hit from which location.[[[New in version 1.2:][Since version 1.2 the model supports the optional output of a grid with the material flux. This requires a 'Material' grid as input. The grid shows the height of the material that has passed in total through each grid cell.[[[Version 1.3:][Version 1.3 includes a fix for material deposition along the process path (amounts and the update of available material for subsequent runs) and improves the output of material flux.[[[Version 1.4:][Since version 1.4 two separate 'Endangered Objects' output grids are created, one encoding the process path cells, the other only the source cells from which objects have been hit. These optional output parameters also have been renamed ('HAZARD_PATHS', 'HAZARD_SOURCES') to improve the legibility of the parameter interface. A third grid, showing the total amount of material in each source cell that has hit objects from that position, has also been added. This output requires a material grid as input in order to calculate material flux. To determine the total amount, the highest material flux observed to hit an object per path and iteration is summed up and converted back to a material height per cell (comparable to the amount specified in the material input grid.)\n
    Arguments
    ----------
    - DEM [`input grid`] : DEM. Digital elevation model [m].
    - RELEASE_AREAS [`input grid`] : Release Areas. Release areas encoded by unique integer IDs, all other cells NoData [-].
    - MATERIAL [`optional input grid`] : Material. Height of material available in each start cell [m].
    - FRICTION_ANGLE_GRID [`optional input grid`] : Friction Angle. Spatially distributed friction angles [degree]. Optionally used with the Geometric Gradient, Fahrboeschung's angle or Shadow Angle friction model.
    - SLOPE_IMPACT_GRID [`optional input grid`] : Slope Impact Areas. Slope impact grid, impact areas encoded with valid values, all other NoData. Optionally used with the Shadow Angle or the 1-parameter friction model.
    - FRICTION_MU_GRID [`optional input grid`] : Friction Parameter Mu. Spatially distributed friction parameter mu [-], optionally used with the 1-parameter friction model or the PCM Model.
    - FRICTION_MASS_TO_DRAG_GRID [`optional input grid`] : Mass to Drag Ratio. Spatially distributed mass to drag ratio [m], optionally used with the PCM Model.
    - OBJECTS [`optional input grid`] : Objects. Potentially endangered objects (like infrastructure) to monitor, using one-hot categorical data encoding for each object class [1, 10, 100, 1000, ...].
    - PROCESS_AREA [`output grid`] : Process Area. Delineated process area with encoded transition frequencies [count].
    - DEPOSITION [`output grid`] : Deposition. Height of the material deposited in each cell [m]. Optional output in case a grid with material amounts is provided as input.
    - MAX_VELOCITY [`output grid`] : Maximum Velocity. Maximum velocity observed in each cell [m/s]. Optional output of the 1-parameter friction model and the PCM Model.
    - STOP_POSITIONS [`output grid`] : Stopping Positions. Stopping positions, showing cells in which the run-out length has been reached [count].
    - HAZARD_PATHS [`output grid`] : Hazard Paths. Process path cells from which objects were hit. Cell values indicate which object classes were hit [combination of object classes]. Optional output in case a grid with potentially endangered objects is provided as input.
    - HAZARD_SOURCES [`output grid`] : Hazard Sources. Source (release area) cells from which objects were hit. Cell values indicate which object classes were hit [combination of object classes]. Optional output in case a grid with potentially endangered objects is provided as input.
    - HAZARD_SOURCES_MATERIAL [`output grid`] : Hazard Sources Material. Source (release area) cells from which objects were hit. Cell values indicate the material amount [m/cell] that has hit objects from that source cell. Optional output in case grids with material amounts and potentially endangered objects are provided as input.
    - MATERIAL_FLUX [`output grid`] : Material Flux. Amount of material that has passed through each cell [m]. Optional output in case a grid with material amounts is provided as input.
    - PROCESS_PATH_MODEL [`choice`] : Model. Available Choices: [0] Maximum Slope [1] Random Walk Default: 1 Choose a process path model.
    - RW_SLOPE_THRES [`floating point number`] : Slope Threshold. Minimum: 0.001000 Maximum: 90.000000 Default: 40.000000 In case the local slope is greater as this threshold [degree], no lateral spreading is modeled.
    - RW_EXPONENT [`floating point number`] : Exponent. Minimum: 1.000000 Default: 2.000000 The exponent [-] is controlling the amount of lateral spreading in case the local slope is in between zero and the slope threshold.
    - RW_PERSISTENCE [`floating point number`] : Persistence Factor. Minimum: 1.000000 Default: 1.500000 Factor [-] used as weight for the current flow direction. A higher factor reduces abrupt changes in flow direction.
    - GPP_ITERATIONS [`integer number`] : Iterations. Minimum: 1 Default: 1000 The number of model runs from each start cell [-].
    - GPP_PROCESSING_ORDER [`choice`] : Processing Order. Available Choices: [0] RAs in Sequence [1] RAs in Sequence per Iteration [2] RAs in Parallel per Iteration Default: 2 Choose the processing order.
    - GPP_SEED [`integer number`] : Seed Value. Minimum: 1 Default: 1 The seed value used to initialize the pseudo-random number generator. A value of 1 will initialize the generator with the current time, higher numbers will always produce the same succession of values for each seed value [-].
    - FRICTION_MODEL [`choice`] : Model. Available Choices: [0] None [1] Geometric Gradient (Heim 1932) [2] Fahrboeschung Principle (Heim 1932) [3] Shadow Angle (Evans & Hungr 1988) [4] 1-parameter friction model (Scheidegger 1975) [5] PCM Model (Perla et al. 1980) Default: 0 Choose a friction model.
    - FRICTION_THRES_FREE_FALL [`floating point number`] : Threshold Angle Free Fall. Minimum: 0.000000 Default: 60.000000 The minimum slope angle [degree] between start cell and current cell for modeling free fall with the Shadow Angle or the 1-parameter friction model.
    - FRICTION_METHOD_IMPACT [`choice`] : Method Impact. Available Choices: [0] Energy Reduction (Scheidegger 1975) [1] Preserved Component of Velocity (Kirkby & Statham 1975) Default: 0 Choose the velocity calculation on slope impact with the 1-parameter friction model.
    - FRICTION_IMPACT_REDUCTION [`floating point number`] : Reduction. Minimum: 0.000000 Maximum: 100.000000 Default: 75.000000 The energy reduction [%] on slope impact with the 1-parameter friction model.
    - FRICTION_ANGLE [`floating point number`] : Friction Angle. Minimum: 0.000000 Maximum: 90.000000 Default: 30.000000 Friction angle [degree] used as Geometric Gradient, Fahrboeschung's angle or Shadow Angle.
    - FRICTION_MU [`floating point number`] : Mu. Minimum: 0.000000 Default: 0.250000 The (constant) friction parameter mu [-] used with the 1-parameter friction model or the PCM Model.
    - FRICTION_MODE_OF_MOTION [`choice`] : Mode of Motion. Available Choices: [0] Sliding [1] Rolling Default: 0 Choose the mode of motion on hillslope with the 1-parameter friction model.
    - FRICTION_MASS_TO_DRAG [`floating point number`] : Mass to Drag Ratio. Minimum: 0.000000 Default: 200.000000 The (constant) mass to drag ratio [m] used with the PCM Model.
    - FRICTION_INIT_VELOCITY [`floating point number`] : Initial Velocity. Minimum: 0.000000 Default: 1.000000 The initial velocity [m/s] used with the PCM Model.
    - DEPOSITION_MODEL [`choice`] : Model. Available Choices: [0] None [1] On Stop [2] Slope & On Stop [3] Velocity & On Stop [4] min(Slope,Velocity) & On Stop Default: 0 Choose a deposition model.
    - DEPOSITION_INITIAL [`floating point number`] : Initial Deposition on Stop. Minimum: 0.000000 Maximum: 100.000000 Default: 20.000000 The percentage of available material (per run) initially deposited at the stopping position [%].
    - DEPOSITION_SLOPE_THRES [`floating point number`] : Slope Threshold. Minimum: 0.000000 Maximum: 90.000000 Default: 20.000000 The slope angle below which the deposition of material is starting [degree].
    - DEPOSITION_VELOCITY_THRES [`floating point number`] : Velocity Threshold. Minimum: 0.000000 Default: 15.000000 The velocity below which the deposition of material is starting [m/s].
    - DEPOSITION_MAX [`floating point number`] : Maximum Deposition along Path. Minimum: 0.000000 Maximum: 100.000000 Default: 20.000000 The percentage of available material (per run) which is deposited at most (slope or velocity equal zero) [%].
    - DEPOSITION_MIN_PATH [`floating point number`] : Minimum Path Length. Minimum: 0.000000 Default: 100.000000 The minimum path length which has to be reached before material deposition is enabled [m].
    - SINK_MIN_SLOPE [`floating point number`] : Minimum Slope. Minimum: 0.000000 Maximum: 90.000000 Default: 2.500000 The minimum slope to preserve on sink filling [degree].

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_geomorphology', '0', 'Gravitational Process Path Model')
    if Tool.is_Okay():
        Tool.Set_Input ('DEM', DEM)
        Tool.Set_Input ('RELEASE_AREAS', RELEASE_AREAS)
        Tool.Set_Input ('MATERIAL', MATERIAL)
        Tool.Set_Input ('FRICTION_ANGLE_GRID', FRICTION_ANGLE_GRID)
        Tool.Set_Input ('SLOPE_IMPACT_GRID', SLOPE_IMPACT_GRID)
        Tool.Set_Input ('FRICTION_MU_GRID', FRICTION_MU_GRID)
        Tool.Set_Input ('FRICTION_MASS_TO_DRAG_GRID', FRICTION_MASS_TO_DRAG_GRID)
        Tool.Set_Input ('OBJECTS', OBJECTS)
        Tool.Set_Output('PROCESS_AREA', PROCESS_AREA)
        Tool.Set_Output('DEPOSITION', DEPOSITION)
        Tool.Set_Output('MAX_VELOCITY', MAX_VELOCITY)
        Tool.Set_Output('STOP_POSITIONS', STOP_POSITIONS)
        Tool.Set_Output('HAZARD_PATHS', HAZARD_PATHS)
        Tool.Set_Output('HAZARD_SOURCES', HAZARD_SOURCES)
        Tool.Set_Output('HAZARD_SOURCES_MATERIAL', HAZARD_SOURCES_MATERIAL)
        Tool.Set_Output('MATERIAL_FLUX', MATERIAL_FLUX)
        Tool.Set_Option('PROCESS_PATH_MODEL', PROCESS_PATH_MODEL)
        Tool.Set_Option('RW_SLOPE_THRES', RW_SLOPE_THRES)
        Tool.Set_Option('RW_EXPONENT', RW_EXPONENT)
        Tool.Set_Option('RW_PERSISTENCE', RW_PERSISTENCE)
        Tool.Set_Option('GPP_ITERATIONS', GPP_ITERATIONS)
        Tool.Set_Option('GPP_PROCESSING_ORDER', GPP_PROCESSING_ORDER)
        Tool.Set_Option('GPP_SEED', GPP_SEED)
        Tool.Set_Option('FRICTION_MODEL', FRICTION_MODEL)
        Tool.Set_Option('FRICTION_THRES_FREE_FALL', FRICTION_THRES_FREE_FALL)
        Tool.Set_Option('FRICTION_METHOD_IMPACT', FRICTION_METHOD_IMPACT)
        Tool.Set_Option('FRICTION_IMPACT_REDUCTION', FRICTION_IMPACT_REDUCTION)
        Tool.Set_Option('FRICTION_ANGLE', FRICTION_ANGLE)
        Tool.Set_Option('FRICTION_MU', FRICTION_MU)
        Tool.Set_Option('FRICTION_MODE_OF_MOTION', FRICTION_MODE_OF_MOTION)
        Tool.Set_Option('FRICTION_MASS_TO_DRAG', FRICTION_MASS_TO_DRAG)
        Tool.Set_Option('FRICTION_INIT_VELOCITY', FRICTION_INIT_VELOCITY)
        Tool.Set_Option('DEPOSITION_MODEL', DEPOSITION_MODEL)
        Tool.Set_Option('DEPOSITION_INITIAL', DEPOSITION_INITIAL)
        Tool.Set_Option('DEPOSITION_SLOPE_THRES', DEPOSITION_SLOPE_THRES)
        Tool.Set_Option('DEPOSITION_VELOCITY_THRES', DEPOSITION_VELOCITY_THRES)
        Tool.Set_Option('DEPOSITION_MAX', DEPOSITION_MAX)
        Tool.Set_Option('DEPOSITION_MIN_PATH', DEPOSITION_MIN_PATH)
        Tool.Set_Option('SINK_MIN_SLOPE', SINK_MIN_SLOPE)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Imagery
- Name     : Segmentation
- ID       : imagery_segmentation

Description
----------
Image segmentation algorithms.
'''

from PySAGA.helper import Tool_Wrapper

def Watershed_Segmentation(
    GRID=None,
    SEGMENTS=None,
    SEEDS=None,
    BORDERS=None,
    OUTPUT=None,
    DOWN=None,
    JOIN=None,
    THRESHOLD=None,
    EDGE=None,
    BBORDERS=None, Verbose=2):
    '''
    Watershed Segmentation
    ----------
    [imagery_segmentation.0]\n
    Watershed segmentation.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - SEGMENTS [`output grid`] : Segments
    - SEEDS [`output shapes`] : Seed Points
    - BORDERS [`output data object`] : Borders
    - OUTPUT [`choice`] : Output. Available Choices: [0] Seed Value [1] Segment ID Default: 1 The values of the resultant grid can be either the seed value (e.g. the local maximum) or the enumerated segment id.
    - DOWN [`choice`] : Method. Available Choices: [0] Minima [1] Maxima Default: 1 Choose if you want to segmentate either on minima or on maxima.
    - JOIN [`choice`] : Join Segments based on Threshold Value. Available Choices: [0] do not join [1] seed to saddle difference [2] seeds difference Default: 0 Join segments based on threshold value.
    - THRESHOLD [`floating point number`] : Threshold. Minimum: 0.000000 Default: 0.000000 Specify a threshold value as minimum difference between neighboured segments.
    - EDGE [`boolean`] : Allow Edge Pixels to be Seeds. Default: 1
    - BBORDERS [`boolean`] : Borders. Default: 0 Create borders between segments as new grid.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '0', 'Watershed Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('SEEDS', SEEDS)
        Tool.Set_Output('BORDERS', BORDERS)
        Tool.Set_Option('OUTPUT', OUTPUT)
        Tool.Set_Option('DOWN', DOWN)
        Tool.Set_Option('JOIN', JOIN)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('EDGE', EDGE)
        Tool.Set_Option('BBORDERS', BBORDERS)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_0(
    GRID=None,
    SEGMENTS=None,
    SEEDS=None,
    BORDERS=None,
    OUTPUT=None,
    DOWN=None,
    JOIN=None,
    THRESHOLD=None,
    EDGE=None,
    BBORDERS=None, Verbose=2):
    '''
    Watershed Segmentation
    ----------
    [imagery_segmentation.0]\n
    Watershed segmentation.\n
    Arguments
    ----------
    - GRID [`input grid`] : Grid
    - SEGMENTS [`output grid`] : Segments
    - SEEDS [`output shapes`] : Seed Points
    - BORDERS [`output data object`] : Borders
    - OUTPUT [`choice`] : Output. Available Choices: [0] Seed Value [1] Segment ID Default: 1 The values of the resultant grid can be either the seed value (e.g. the local maximum) or the enumerated segment id.
    - DOWN [`choice`] : Method. Available Choices: [0] Minima [1] Maxima Default: 1 Choose if you want to segmentate either on minima or on maxima.
    - JOIN [`choice`] : Join Segments based on Threshold Value. Available Choices: [0] do not join [1] seed to saddle difference [2] seeds difference Default: 0 Join segments based on threshold value.
    - THRESHOLD [`floating point number`] : Threshold. Minimum: 0.000000 Default: 0.000000 Specify a threshold value as minimum difference between neighboured segments.
    - EDGE [`boolean`] : Allow Edge Pixels to be Seeds. Default: 1
    - BBORDERS [`boolean`] : Borders. Default: 0 Create borders between segments as new grid.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '0', 'Watershed Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('GRID', GRID)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('SEEDS', SEEDS)
        Tool.Set_Output('BORDERS', BORDERS)
        Tool.Set_Option('OUTPUT', OUTPUT)
        Tool.Set_Option('DOWN', DOWN)
        Tool.Set_Option('JOIN', JOIN)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('EDGE', EDGE)
        Tool.Set_Option('BBORDERS', BBORDERS)
        return Tool.Execute(Verbose)
    return False

def Grid_Skeletonization(
    INPUT=None,
    RESULT=None,
    VECTOR=None,
    METHOD=None,
    INIT_METHOD=None,
    INIT_THRESHOLD=None,
    CONVERGENCE=None, Verbose=2):
    '''
    Grid Skeletonization
    ----------
    [imagery_segmentation.1]\n
    Simple skeletonisation methods for grids.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - RESULT [`output grid`] : Skeleton
    - VECTOR [`output shapes`] : Skeleton
    - METHOD [`choice`] : Method. Available Choices: [0] Standard [1] Hilditch's Algorithm [2] Channel Skeleton Default: 0
    - INIT_METHOD [`choice`] : Initialisation. Available Choices: [0] less than [1] greater than [2] less than or equal [3] greater than or equal Default: 1 Initialize a grid cell as potential skeleton member based on given threshold.
    - INIT_THRESHOLD [`floating point number`] : Threshold. Default: 0.000000
    - CONVERGENCE [`integer number`] : Convergence. Minimum: 0 Default: 3

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '1', 'Grid Skeletonization')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Output('VECTOR', VECTOR)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('INIT_METHOD', INIT_METHOD)
        Tool.Set_Option('INIT_THRESHOLD', INIT_THRESHOLD)
        Tool.Set_Option('CONVERGENCE', CONVERGENCE)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_1(
    INPUT=None,
    RESULT=None,
    VECTOR=None,
    METHOD=None,
    INIT_METHOD=None,
    INIT_THRESHOLD=None,
    CONVERGENCE=None, Verbose=2):
    '''
    Grid Skeletonization
    ----------
    [imagery_segmentation.1]\n
    Simple skeletonisation methods for grids.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - RESULT [`output grid`] : Skeleton
    - VECTOR [`output shapes`] : Skeleton
    - METHOD [`choice`] : Method. Available Choices: [0] Standard [1] Hilditch's Algorithm [2] Channel Skeleton Default: 0
    - INIT_METHOD [`choice`] : Initialisation. Available Choices: [0] less than [1] greater than [2] less than or equal [3] greater than or equal Default: 1 Initialize a grid cell as potential skeleton member based on given threshold.
    - INIT_THRESHOLD [`floating point number`] : Threshold. Default: 0.000000
    - CONVERGENCE [`integer number`] : Convergence. Minimum: 0 Default: 3

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '1', 'Grid Skeletonization')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Output('VECTOR', VECTOR)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('INIT_METHOD', INIT_METHOD)
        Tool.Set_Option('INIT_THRESHOLD', INIT_THRESHOLD)
        Tool.Set_Option('CONVERGENCE', CONVERGENCE)
        return Tool.Execute(Verbose)
    return False

def Seed_Generation(
    FEATURES=None,
    VARIANCE=None,
    SEED_GRID=None,
    SEED_POINTS=None,
    SEED_TYPE=None,
    METHOD=None,
    BAND_WIDTH=None,
    NORMALIZE=None,
    DW_WEIGHTING=None,
    DW_IDW_POWER=None,
    DW_BANDWIDTH=None, Verbose=2):
    '''
    Seed Generation
    ----------
    [imagery_segmentation.2]\n
    The tool allows one to create seed points from a stack of input features. Such seed points can be used, for example, as input in the 'Seeded Region Growing' tool.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - VARIANCE [`output grid`] : Variance
    - SEED_GRID [`output grid`] : Seeds Grid
    - SEED_POINTS [`output shapes`] : Seed Points
    - SEED_TYPE [`choice`] : Seed Type. Available Choices: [0] minima of variance [1] maxima of variance Default: 0
    - METHOD [`choice`] : Method. Available Choices: [0] band width smoothing [1] band width search Default: 0
    - BAND_WIDTH [`floating point number`] : Bandwidth (Cells). Minimum: 1.000000 Default: 10.000000
    - NORMALIZE [`boolean`] : Normalize Features. Default: 0 Standardize the input features, i.e. rescale the input data (features) such that the mean equals 0 and the standard deviation equals 1. This is helpful when the input features have different scales, units or outliers.
    - DW_WEIGHTING [`choice`] : Weighting Function. Available Choices: [0] no distance weighting [1] inverse distance to a power [2] exponential [3] gaussian Default: 3
    - DW_IDW_POWER [`floating point number`] : Power. Minimum: 0.000000 Default: 2.000000
    - DW_BANDWIDTH [`floating point number`] : Bandwidth. Minimum: 0.000000 Default: 5.000000 Bandwidth for exponential and Gaussian weighting

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '2', 'Seed Generation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('SEED_GRID', SEED_GRID)
        Tool.Set_Output('SEED_POINTS', SEED_POINTS)
        Tool.Set_Option('SEED_TYPE', SEED_TYPE)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('BAND_WIDTH', BAND_WIDTH)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('DW_WEIGHTING', DW_WEIGHTING)
        Tool.Set_Option('DW_IDW_POWER', DW_IDW_POWER)
        Tool.Set_Option('DW_BANDWIDTH', DW_BANDWIDTH)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_2(
    FEATURES=None,
    VARIANCE=None,
    SEED_GRID=None,
    SEED_POINTS=None,
    SEED_TYPE=None,
    METHOD=None,
    BAND_WIDTH=None,
    NORMALIZE=None,
    DW_WEIGHTING=None,
    DW_IDW_POWER=None,
    DW_BANDWIDTH=None, Verbose=2):
    '''
    Seed Generation
    ----------
    [imagery_segmentation.2]\n
    The tool allows one to create seed points from a stack of input features. Such seed points can be used, for example, as input in the 'Seeded Region Growing' tool.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - VARIANCE [`output grid`] : Variance
    - SEED_GRID [`output grid`] : Seeds Grid
    - SEED_POINTS [`output shapes`] : Seed Points
    - SEED_TYPE [`choice`] : Seed Type. Available Choices: [0] minima of variance [1] maxima of variance Default: 0
    - METHOD [`choice`] : Method. Available Choices: [0] band width smoothing [1] band width search Default: 0
    - BAND_WIDTH [`floating point number`] : Bandwidth (Cells). Minimum: 1.000000 Default: 10.000000
    - NORMALIZE [`boolean`] : Normalize Features. Default: 0 Standardize the input features, i.e. rescale the input data (features) such that the mean equals 0 and the standard deviation equals 1. This is helpful when the input features have different scales, units or outliers.
    - DW_WEIGHTING [`choice`] : Weighting Function. Available Choices: [0] no distance weighting [1] inverse distance to a power [2] exponential [3] gaussian Default: 3
    - DW_IDW_POWER [`floating point number`] : Power. Minimum: 0.000000 Default: 2.000000
    - DW_BANDWIDTH [`floating point number`] : Bandwidth. Minimum: 0.000000 Default: 5.000000 Bandwidth for exponential and Gaussian weighting

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '2', 'Seed Generation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('VARIANCE', VARIANCE)
        Tool.Set_Output('SEED_GRID', SEED_GRID)
        Tool.Set_Output('SEED_POINTS', SEED_POINTS)
        Tool.Set_Option('SEED_TYPE', SEED_TYPE)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('BAND_WIDTH', BAND_WIDTH)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('DW_WEIGHTING', DW_WEIGHTING)
        Tool.Set_Option('DW_IDW_POWER', DW_IDW_POWER)
        Tool.Set_Option('DW_BANDWIDTH', DW_BANDWIDTH)
        return Tool.Execute(Verbose)
    return False

def Seeded_Region_Growing(
    SEEDS=None,
    FEATURES=None,
    SEGMENTS=None,
    SIMILARITY=None,
    TABLE=None,
    NORMALIZE=None,
    NEIGHBOUR=None,
    METHOD=None,
    SIG_1=None,
    SIG_2=None,
    THRESHOLD=None,
    REFRESH=None,
    LEAFSIZE=None, Verbose=2):
    '''
    Seeded Region Growing
    ----------
    [imagery_segmentation.3]\n
    The tool allows one to apply a seeded region growing algorithm to a stack of input features and thus to segmentize the data for object extraction. The required seed points can be created with the 'Seed Generation' tool, for example. The derived segments can be used, for example, for object based classification.\n
    Arguments
    ----------
    - SEEDS [`input grid`] : Seeds
    - FEATURES [`input grid list`] : Features
    - SEGMENTS [`output grid`] : Segments
    - SIMILARITY [`output grid`] : Similarity
    - TABLE [`output table`] : Seeds
    - NORMALIZE [`boolean`] : Normalize Features. Default: 0 Standardize the input features, i.e. rescale the input data (features) such that the mean equals 0 and the standard deviation equals 1. This is helpful when the input features have different scales, units or outliers.
    - NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (von Neumann) [1] 8 (Moore) Default: 0
    - METHOD [`choice`] : Method. Available Choices: [0] feature space and position [1] feature space Default: 0
    - SIG_1 [`floating point number`] : Variance in Feature Space. Minimum: 0.000000 Default: 1.000000
    - SIG_2 [`floating point number`] : Variance in Position Space. Minimum: 0.000000 Default: 1.000000
    - THRESHOLD [`floating point number`] : Similarity Threshold. Minimum: 0.000000 Default: 0.000000
    - REFRESH [`boolean`] : Refresh. Default: 0
    - LEAFSIZE [`integer number`] : Leaf Size (for Speed Optimisation). Minimum: 2 Default: 256

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '3', 'Seeded Region Growing')
    if Tool.is_Okay():
        Tool.Set_Input ('SEEDS', SEEDS)
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('SIMILARITY', SIMILARITY)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('NEIGHBOUR', NEIGHBOUR)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('SIG_1', SIG_1)
        Tool.Set_Option('SIG_2', SIG_2)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('LEAFSIZE', LEAFSIZE)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_3(
    SEEDS=None,
    FEATURES=None,
    SEGMENTS=None,
    SIMILARITY=None,
    TABLE=None,
    NORMALIZE=None,
    NEIGHBOUR=None,
    METHOD=None,
    SIG_1=None,
    SIG_2=None,
    THRESHOLD=None,
    REFRESH=None,
    LEAFSIZE=None, Verbose=2):
    '''
    Seeded Region Growing
    ----------
    [imagery_segmentation.3]\n
    The tool allows one to apply a seeded region growing algorithm to a stack of input features and thus to segmentize the data for object extraction. The required seed points can be created with the 'Seed Generation' tool, for example. The derived segments can be used, for example, for object based classification.\n
    Arguments
    ----------
    - SEEDS [`input grid`] : Seeds
    - FEATURES [`input grid list`] : Features
    - SEGMENTS [`output grid`] : Segments
    - SIMILARITY [`output grid`] : Similarity
    - TABLE [`output table`] : Seeds
    - NORMALIZE [`boolean`] : Normalize Features. Default: 0 Standardize the input features, i.e. rescale the input data (features) such that the mean equals 0 and the standard deviation equals 1. This is helpful when the input features have different scales, units or outliers.
    - NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (von Neumann) [1] 8 (Moore) Default: 0
    - METHOD [`choice`] : Method. Available Choices: [0] feature space and position [1] feature space Default: 0
    - SIG_1 [`floating point number`] : Variance in Feature Space. Minimum: 0.000000 Default: 1.000000
    - SIG_2 [`floating point number`] : Variance in Position Space. Minimum: 0.000000 Default: 1.000000
    - THRESHOLD [`floating point number`] : Similarity Threshold. Minimum: 0.000000 Default: 0.000000
    - REFRESH [`boolean`] : Refresh. Default: 0
    - LEAFSIZE [`integer number`] : Leaf Size (for Speed Optimisation). Minimum: 2 Default: 256

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '3', 'Seeded Region Growing')
    if Tool.is_Okay():
        Tool.Set_Input ('SEEDS', SEEDS)
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('SEGMENTS', SEGMENTS)
        Tool.Set_Output('SIMILARITY', SIMILARITY)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('NEIGHBOUR', NEIGHBOUR)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('SIG_1', SIG_1)
        Tool.Set_Option('SIG_2', SIG_2)
        Tool.Set_Option('THRESHOLD', THRESHOLD)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('LEAFSIZE', LEAFSIZE)
        return Tool.Execute(Verbose)
    return False

def Superpixel_Segmentation(
    FEATURES=None,
    POLYGONS=None,
    SUPERPIXELS=None,
    NORMALIZE=None,
    MAX_ITERATIONS=None,
    REGULARIZATION=None,
    SIZE=None,
    SIZE_MIN=None,
    SUPERPIXELS_DO=None,
    POSTPROCESSING=None,
    NCLUSTER=None,
    SPLIT_CLUSTERS=None, Verbose=2):
    '''
    Superpixel Segmentation
    ----------
    [imagery_segmentation.4]\n
    The Superpixel Segmentation tool implements the 'Simple Linear Iterative Clustering' (SLIC) algorithm, an image segmentation method described in Achanta et al. (2010).\n
    SLIC is a simple and efficient method to decompose an image in visually homogeneous regions. It is based on a spatially localized version of k-means clustering. Similar to mean shift or quick shift, each pixel is associated to a feature vector.\n
    This tool is follows the SLIC implementation created by Vedaldi and Fulkerson as part of the VLFeat library.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - POLYGONS [`output shapes`] : Segments
    - SUPERPIXELS [`output grid list`] : Superpixels
    - NORMALIZE [`boolean`] : Normalize. Default: 0
    - MAX_ITERATIONS [`integer number`] : Maximum Iterations. Minimum: 1 Default: 100
    - REGULARIZATION [`floating point number`] : Regularization. Minimum: 0.000000 Default: 1.000000
    - SIZE [`integer number`] : Region Size. Minimum: 1 Default: 10 Starting 'cell size' of the superpixels given as number of cells.
    - SIZE_MIN [`integer number`] : Minimum Region Size. Minimum: 1 Default: 1 In postprocessing join segments, which cover less cells than specified here, to a larger neighbour segment.
    - SUPERPIXELS_DO [`boolean`] : Create Superpixel Grids. Default: 0
    - POSTPROCESSING [`choice`] : Post-Processing. Available Choices: [0] none [1] unsupervised classification Default: 0
    - NCLUSTER [`integer number`] : Number of Clusters. Minimum: 2 Default: 12
    - SPLIT_CLUSTERS [`boolean`] : Split Clusters. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '4', 'Superpixel Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('POLYGONS', POLYGONS)
        Tool.Set_Output('SUPERPIXELS', SUPERPIXELS)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('MAX_ITERATIONS', MAX_ITERATIONS)
        Tool.Set_Option('REGULARIZATION', REGULARIZATION)
        Tool.Set_Option('SIZE', SIZE)
        Tool.Set_Option('SIZE_MIN', SIZE_MIN)
        Tool.Set_Option('SUPERPIXELS_DO', SUPERPIXELS_DO)
        Tool.Set_Option('POSTPROCESSING', POSTPROCESSING)
        Tool.Set_Option('NCLUSTER', NCLUSTER)
        Tool.Set_Option('SPLIT_CLUSTERS', SPLIT_CLUSTERS)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_4(
    FEATURES=None,
    POLYGONS=None,
    SUPERPIXELS=None,
    NORMALIZE=None,
    MAX_ITERATIONS=None,
    REGULARIZATION=None,
    SIZE=None,
    SIZE_MIN=None,
    SUPERPIXELS_DO=None,
    POSTPROCESSING=None,
    NCLUSTER=None,
    SPLIT_CLUSTERS=None, Verbose=2):
    '''
    Superpixel Segmentation
    ----------
    [imagery_segmentation.4]\n
    The Superpixel Segmentation tool implements the 'Simple Linear Iterative Clustering' (SLIC) algorithm, an image segmentation method described in Achanta et al. (2010).\n
    SLIC is a simple and efficient method to decompose an image in visually homogeneous regions. It is based on a spatially localized version of k-means clustering. Similar to mean shift or quick shift, each pixel is associated to a feature vector.\n
    This tool is follows the SLIC implementation created by Vedaldi and Fulkerson as part of the VLFeat library.\n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - POLYGONS [`output shapes`] : Segments
    - SUPERPIXELS [`output grid list`] : Superpixels
    - NORMALIZE [`boolean`] : Normalize. Default: 0
    - MAX_ITERATIONS [`integer number`] : Maximum Iterations. Minimum: 1 Default: 100
    - REGULARIZATION [`floating point number`] : Regularization. Minimum: 0.000000 Default: 1.000000
    - SIZE [`integer number`] : Region Size. Minimum: 1 Default: 10 Starting 'cell size' of the superpixels given as number of cells.
    - SIZE_MIN [`integer number`] : Minimum Region Size. Minimum: 1 Default: 1 In postprocessing join segments, which cover less cells than specified here, to a larger neighbour segment.
    - SUPERPIXELS_DO [`boolean`] : Create Superpixel Grids. Default: 0
    - POSTPROCESSING [`choice`] : Post-Processing. Available Choices: [0] none [1] unsupervised classification Default: 0
    - NCLUSTER [`integer number`] : Number of Clusters. Minimum: 2 Default: 12
    - SPLIT_CLUSTERS [`boolean`] : Split Clusters. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '4', 'Superpixel Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Output('POLYGONS', POLYGONS)
        Tool.Set_Output('SUPERPIXELS', SUPERPIXELS)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('MAX_ITERATIONS', MAX_ITERATIONS)
        Tool.Set_Option('REGULARIZATION', REGULARIZATION)
        Tool.Set_Option('SIZE', SIZE)
        Tool.Set_Option('SIZE_MIN', SIZE_MIN)
        Tool.Set_Option('SUPERPIXELS_DO', SUPERPIXELS_DO)
        Tool.Set_Option('POSTPROCESSING', POSTPROCESSING)
        Tool.Set_Option('NCLUSTER', NCLUSTER)
        Tool.Set_Option('SPLIT_CLUSTERS', SPLIT_CLUSTERS)
        return Tool.Execute(Verbose)
    return False

def Connected_Component_Labeling(
    INPUT=None,
    OUTPUT=None,
    NEIGHBOUR=None, Verbose=2):
    '''
    Connected Component Labeling
    ----------
    [imagery_segmentation.5]\n
    The tool allows one to label subsets of connected components with a unique identifier. Connected component labeling (CCL) is an operator which turns a binary image into a symbolic image in which the label assigned to each pixel is an integer uniquely identifying the connected component to which that pixel belongs (Shapiro 1996).\n
    The tool takes a grid as input and treats it as a binary image. The foreground is defined by all cell values greater than zero, the background by NoData cells and all cell values less than one. Connectivity can be determined by analysing either a 4-connected or a 8-connected neighborhood.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. Grid to analyse.
    - OUTPUT [`output grid`] : Output. Output grid with labeled components.
    - NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (von Neumann) [1] 8 (Moore) Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '5', 'Connected Component Labeling')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('NEIGHBOUR', NEIGHBOUR)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_5(
    INPUT=None,
    OUTPUT=None,
    NEIGHBOUR=None, Verbose=2):
    '''
    Connected Component Labeling
    ----------
    [imagery_segmentation.5]\n
    The tool allows one to label subsets of connected components with a unique identifier. Connected component labeling (CCL) is an operator which turns a binary image into a symbolic image in which the label assigned to each pixel is an integer uniquely identifying the connected component to which that pixel belongs (Shapiro 1996).\n
    The tool takes a grid as input and treats it as a binary image. The foreground is defined by all cell values greater than zero, the background by NoData cells and all cell values less than one. Connectivity can be determined by analysing either a 4-connected or a 8-connected neighborhood.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. Grid to analyse.
    - OUTPUT [`output grid`] : Output. Output grid with labeled components.
    - NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (von Neumann) [1] 8 (Moore) Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', '5', 'Connected Component Labeling')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('OUTPUT', OUTPUT)
        Tool.Set_Option('NEIGHBOUR', NEIGHBOUR)
        return Tool.Execute(Verbose)
    return False

def Object_Based_Image_Segmentation(
    FEATURES=None,
    SAMPLES=None,
    OBJECTS=None,
    GRID_SYSTEM=None,
    NORMALIZE=None,
    SEEDS_BAND_WIDTH=None,
    RGA_NEIGHBOUR=None,
    RGA_METHOD=None,
    RGA_SIG_1=None,
    RGA_SIG_2=None,
    RGA_SIMILARITY=None,
    MAJORITY_RADIUS=None,
    CLASSIFICATION=None,
    SPLIT_POLYGONS=None,
    NCLUSTER=None,
    CLASSIFIER=None, Verbose=2):
    '''
    Object Based Image Segmentation
    ----------
    [imagery_segmentation.obia]\n
    This 'Object Based Image Segmentation' tool chain combines a number of tools for an easy derivation of geo-objects as polygons and is typically applied to satellite imagery. Segmentation is done using a 'Seeded Region Growing Algorithm'. Optionally the resulting polygons can be grouped by an unsupervised classification (k-means cluster analysis) or supervised classification (needs classified feature samples as additional input), both is performed on the basis of zonal feature grid statistics for each polygon object.\n
    \n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - SAMPLES [`input table`] : Training Samples. Training samples for supervised classification. Provide a class identifier in the first field followed by sample data corresponding to the selected feature attributes
    - OBJECTS [`output shapes`] : Segments
    - GRID_SYSTEM [`grid system`] : Grid System
    - NORMALIZE [`boolean`] : Normalize. Default: 0
    - SEEDS_BAND_WIDTH [`floating point number`] : Band Width for Seed Point Generation. Default: 2.000000 Increase band width to get less seed points.
    - RGA_NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (Neumann) [1] 8 (Moore) Default: 0
    - RGA_METHOD [`choice`] : Distance. Available Choices: [0] feature space and position [1] feature space Default: 0
    - RGA_SIG_1 [`floating point number`] : Variance in Feature Space. Minimum: 0.000000 Default: 1.000000
    - RGA_SIG_2 [`floating point number`] : Variance in Position Space. Minimum: 0.000000 Default: 1.000000
    - RGA_SIMILARITY [`floating point number`] : Similarity Threshold. Minimum: 0.000000 Default: 0.000000
    - MAJORITY_RADIUS [`integer number`] : Generalization. Default: 1 Applies a majority filter with given search radius to the segments grid. Is skipped if set to zero.
    - CLASSIFICATION [`choice`] : Classification. Available Choices: [0] none [1] cluster analysis [2] supervised classification Default: 0
    - SPLIT_POLYGONS [`choice`] : Split Distinct Polygon Parts. Available Choices: [0] no [1] yes Default: 0
    - NCLUSTER [`integer number`] : Number of Clusters. Default: 10
    - CLASSIFIER [`choice`] : Method. Available Choices: [0] Binary Encoding [1] Parallelepiped [2] Minimum Distance [3] Mahalanobis Distance [4] Maximum Likelihood [5] Spectral Angle Mapping Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', 'obia', 'Object Based Image Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('SAMPLES', SAMPLES)
        Tool.Set_Output('OBJECTS', OBJECTS)
        Tool.Set_Option('GRID_SYSTEM', GRID_SYSTEM)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('SEEDS_BAND_WIDTH', SEEDS_BAND_WIDTH)
        Tool.Set_Option('RGA_NEIGHBOUR', RGA_NEIGHBOUR)
        Tool.Set_Option('RGA_METHOD', RGA_METHOD)
        Tool.Set_Option('RGA_SIG_1', RGA_SIG_1)
        Tool.Set_Option('RGA_SIG_2', RGA_SIG_2)
        Tool.Set_Option('RGA_SIMILARITY', RGA_SIMILARITY)
        Tool.Set_Option('MAJORITY_RADIUS', MAJORITY_RADIUS)
        Tool.Set_Option('CLASSIFICATION', CLASSIFICATION)
        Tool.Set_Option('SPLIT_POLYGONS', SPLIT_POLYGONS)
        Tool.Set_Option('NCLUSTER', NCLUSTER)
        Tool.Set_Option('CLASSIFIER', CLASSIFIER)
        return Tool.Execute(Verbose)
    return False

def run_tool_imagery_segmentation_obia(
    FEATURES=None,
    SAMPLES=None,
    OBJECTS=None,
    GRID_SYSTEM=None,
    NORMALIZE=None,
    SEEDS_BAND_WIDTH=None,
    RGA_NEIGHBOUR=None,
    RGA_METHOD=None,
    RGA_SIG_1=None,
    RGA_SIG_2=None,
    RGA_SIMILARITY=None,
    MAJORITY_RADIUS=None,
    CLASSIFICATION=None,
    SPLIT_POLYGONS=None,
    NCLUSTER=None,
    CLASSIFIER=None, Verbose=2):
    '''
    Object Based Image Segmentation
    ----------
    [imagery_segmentation.obia]\n
    This 'Object Based Image Segmentation' tool chain combines a number of tools for an easy derivation of geo-objects as polygons and is typically applied to satellite imagery. Segmentation is done using a 'Seeded Region Growing Algorithm'. Optionally the resulting polygons can be grouped by an unsupervised classification (k-means cluster analysis) or supervised classification (needs classified feature samples as additional input), both is performed on the basis of zonal feature grid statistics for each polygon object.\n
    \n
    Arguments
    ----------
    - FEATURES [`input grid list`] : Features
    - SAMPLES [`input table`] : Training Samples. Training samples for supervised classification. Provide a class identifier in the first field followed by sample data corresponding to the selected feature attributes
    - OBJECTS [`output shapes`] : Segments
    - GRID_SYSTEM [`grid system`] : Grid System
    - NORMALIZE [`boolean`] : Normalize. Default: 0
    - SEEDS_BAND_WIDTH [`floating point number`] : Band Width for Seed Point Generation. Default: 2.000000 Increase band width to get less seed points.
    - RGA_NEIGHBOUR [`choice`] : Neighbourhood. Available Choices: [0] 4 (Neumann) [1] 8 (Moore) Default: 0
    - RGA_METHOD [`choice`] : Distance. Available Choices: [0] feature space and position [1] feature space Default: 0
    - RGA_SIG_1 [`floating point number`] : Variance in Feature Space. Minimum: 0.000000 Default: 1.000000
    - RGA_SIG_2 [`floating point number`] : Variance in Position Space. Minimum: 0.000000 Default: 1.000000
    - RGA_SIMILARITY [`floating point number`] : Similarity Threshold. Minimum: 0.000000 Default: 0.000000
    - MAJORITY_RADIUS [`integer number`] : Generalization. Default: 1 Applies a majority filter with given search radius to the segments grid. Is skipped if set to zero.
    - CLASSIFICATION [`choice`] : Classification. Available Choices: [0] none [1] cluster analysis [2] supervised classification Default: 0
    - SPLIT_POLYGONS [`choice`] : Split Distinct Polygon Parts. Available Choices: [0] no [1] yes Default: 0
    - NCLUSTER [`integer number`] : Number of Clusters. Default: 10
    - CLASSIFIER [`choice`] : Method. Available Choices: [0] Binary Encoding [1] Parallelepiped [2] Minimum Distance [3] Mahalanobis Distance [4] Maximum Likelihood [5] Spectral Angle Mapping Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('imagery_segmentation', 'obia', 'Object Based Image Segmentation')
    if Tool.is_Okay():
        Tool.Set_Input ('FEATURES', FEATURES)
        Tool.Set_Input ('SAMPLES', SAMPLES)
        Tool.Set_Output('OBJECTS', OBJECTS)
        Tool.Set_Option('GRID_SYSTEM', GRID_SYSTEM)
        Tool.Set_Option('NORMALIZE', NORMALIZE)
        Tool.Set_Option('SEEDS_BAND_WIDTH', SEEDS_BAND_WIDTH)
        Tool.Set_Option('RGA_NEIGHBOUR', RGA_NEIGHBOUR)
        Tool.Set_Option('RGA_METHOD', RGA_METHOD)
        Tool.Set_Option('RGA_SIG_1', RGA_SIG_1)
        Tool.Set_Option('RGA_SIG_2', RGA_SIG_2)
        Tool.Set_Option('RGA_SIMILARITY', RGA_SIMILARITY)
        Tool.Set_Option('MAJORITY_RADIUS', MAJORITY_RADIUS)
        Tool.Set_Option('CLASSIFICATION', CLASSIFICATION)
        Tool.Set_Option('SPLIT_POLYGONS', SPLIT_POLYGONS)
        Tool.Set_Option('NCLUSTER', NCLUSTER)
        Tool.Set_Option('CLASSIFIER', CLASSIFIER)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Grid
- Name     : Filter (Perego 2009)
- ID       : contrib_perego

Description
----------
Contributions from Alessandro Perego. Go to [www.alspergis.altervista.org](http://www.alspergis.altervista.org/software/software.html#saga) for further information.
'''

from PySAGA.helper import Tool_Wrapper

def Average_With_Threshold_1(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 1
    ----------
    [contrib_perego.0]\n
    Average With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. It's useful to remove noise with a known maximum reducing the loss of information\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A1WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '0', 'Average With Threshold 1')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_0(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 1
    ----------
    [contrib_perego.0]\n
    Average With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. It's useful to remove noise with a known maximum reducing the loss of information\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A1WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '0', 'Average With Threshold 1')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def Average_With_Threshold_2(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 2
    ----------
    [contrib_perego.1]\n
    Average 2 With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. Each value has a weight which is inversely proportional to the distance (method 1).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A2WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '1', 'Average With Threshold 2')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_1(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 2
    ----------
    [contrib_perego.1]\n
    Average 2 With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. Each value has a weight which is inversely proportional to the distance (method 1).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A2WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '1', 'Average With Threshold 2')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def Average_With_Threshold_3(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 3
    ----------
    [contrib_perego.2]\n
    Average 3 With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. Each value has a weight which is inversely proportional to the distance (method 2).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A3WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '2', 'Average With Threshold 3')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_2(
    INPUT=None,
    RESULT=None,
    RX=None,
    RY=None,
    THRESH=None, Verbose=2):
    '''
    Average With Threshold 3
    ----------
    [contrib_perego.2]\n
    Average 3 With Threshold for Grids calculates average in X and Y distances using only the values that differ form central pixel less than a specified threshold. Each value has a weight which is inversely proportional to the distance (method 2).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : AWT Grid. New grid filtered with the A3WiTh tool
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1
    - THRESH [`floating point number`] : Threshold. Default: 2.000000 The values in the specified radius is used in the average calculation only if its difference with the central value is lesser or equal to this threshold.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '2', 'Average With Threshold 3')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        Tool.Set_Option('THRESH', THRESH)
        return Tool.Execute(Verbose)
    return False

def Average_With_Mask_1(
    INPUT=None,
    MASK=None,
    RESULT=None,
    V=None,
    RX=None,
    RY=None, Verbose=2):
    '''
    Average With Mask 1
    ----------
    [contrib_perego.3]\n
    Average With Mask 1 calculates average for cells specified by a mask grid. Cell excluded by the mask grid are NOT used in the average calculation.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - MASK [`input grid`] : Mask Grid. This grid indicates the cells you want calculate the average.
    - RESULT [`output grid`] : AWM1 Grid. New grid filtered with the AvWiMa1 tool
    - V [`floating point number`] : Mask value. Default: 1.000000 Value of right cells in the Mask Grid
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '3', 'Average With Mask 1')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('V', V)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_3(
    INPUT=None,
    MASK=None,
    RESULT=None,
    V=None,
    RX=None,
    RY=None, Verbose=2):
    '''
    Average With Mask 1
    ----------
    [contrib_perego.3]\n
    Average With Mask 1 calculates average for cells specified by a mask grid. Cell excluded by the mask grid are NOT used in the average calculation.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - MASK [`input grid`] : Mask Grid. This grid indicates the cells you want calculate the average.
    - RESULT [`output grid`] : AWM1 Grid. New grid filtered with the AvWiMa1 tool
    - V [`floating point number`] : Mask value. Default: 1.000000 Value of right cells in the Mask Grid
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '3', 'Average With Mask 1')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('V', V)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        return Tool.Execute(Verbose)
    return False

def Average_With_Mask_2(
    INPUT=None,
    MASK=None,
    RESULT=None,
    V=None,
    RX=None,
    RY=None, Verbose=2):
    '''
    Average With Mask 2
    ----------
    [contrib_perego.4]\n
    Average With Mask 2 calculates average for cells specified by a mask grid. However cell excluded by the mask grid are used in the average calculation for right pixels.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - MASK [`input grid`] : Mask Grid. This grid indicates the cells you want calculate the average.
    - RESULT [`output grid`] : AWM2 Grid. New grid filtered with the AvWiMa2 tool
    - V [`floating point number`] : Mask value. Default: 1.000000 Value of right cells in the Mask Grid
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '4', 'Average With Mask 2')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('V', V)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_4(
    INPUT=None,
    MASK=None,
    RESULT=None,
    V=None,
    RX=None,
    RY=None, Verbose=2):
    '''
    Average With Mask 2
    ----------
    [contrib_perego.4]\n
    Average With Mask 2 calculates average for cells specified by a mask grid. However cell excluded by the mask grid are used in the average calculation for right pixels.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - MASK [`input grid`] : Mask Grid. This grid indicates the cells you want calculate the average.
    - RESULT [`output grid`] : AWM2 Grid. New grid filtered with the AvWiMa2 tool
    - V [`floating point number`] : Mask value. Default: 1.000000 Value of right cells in the Mask Grid
    - RX [`integer number`] : Radius X. Minimum: 1 Default: 1
    - RY [`integer number`] : Radius Y. Minimum: 1 Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '4', 'Average With Mask 2')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('V', V)
        Tool.Set_Option('RX', RX)
        Tool.Set_Option('RY', RY)
        return Tool.Execute(Verbose)
    return False

def Destriping(
    INPUT=None,
    MASK=None,
    RESULT3=None,
    RESULT1=None,
    RESULT2=None,
    STRIPES=None,
    ANG=None,
    R=None,
    D=None, Verbose=2):
    '''
    Destriping
    ----------
    [contrib_perego.5]\n
    Destriping filter removes straight parallel stripes in raster data. It uses two low-pass filters elongated in the stripes direction; the first one is 1 pixel unit wide while the second one is wide as the striping wavelength. Their difference is the striping error which is removed from the original data to obtain the destriped DEM. This method is equivalent to that proposed by Oimoen (2000).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - MASK [`optional input grid`] : Mask. Mask's no-data cells will be excluded from processing
    - RESULT3 [`output grid`] : Destriped Grid
    - RESULT1 [`output grid`] : Low-pass 1. Step 1: low-pass of stripe
    - RESULT2 [`output grid`] : Low-pass 2. Step 2: low-pass between stripe and its surroundings
    - STRIPES [`output grid`] : Stripes. The difference between destriped and original grid
    - ANG [`floating point number`] : Angle. Default: 0.000000 [Degree], 0 = horizontal, 90 = vertical
    - R [`floating point number`] : Radius. Minimum: 1.000000 Default: 20.000000 [Cells]
    - D [`floating point number`] : Stripes Distance. Minimum: 2.000000 Default: 2.000000 [Cells]

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '5', 'Destriping')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT3', RESULT3)
        Tool.Set_Output('RESULT1', RESULT1)
        Tool.Set_Output('RESULT2', RESULT2)
        Tool.Set_Output('STRIPES', STRIPES)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R', R)
        Tool.Set_Option('D', D)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_5(
    INPUT=None,
    MASK=None,
    RESULT3=None,
    RESULT1=None,
    RESULT2=None,
    STRIPES=None,
    ANG=None,
    R=None,
    D=None, Verbose=2):
    '''
    Destriping
    ----------
    [contrib_perego.5]\n
    Destriping filter removes straight parallel stripes in raster data. It uses two low-pass filters elongated in the stripes direction; the first one is 1 pixel unit wide while the second one is wide as the striping wavelength. Their difference is the striping error which is removed from the original data to obtain the destriped DEM. This method is equivalent to that proposed by Oimoen (2000).\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - MASK [`optional input grid`] : Mask. Mask's no-data cells will be excluded from processing
    - RESULT3 [`output grid`] : Destriped Grid
    - RESULT1 [`output grid`] : Low-pass 1. Step 1: low-pass of stripe
    - RESULT2 [`output grid`] : Low-pass 2. Step 2: low-pass between stripe and its surroundings
    - STRIPES [`output grid`] : Stripes. The difference between destriped and original grid
    - ANG [`floating point number`] : Angle. Default: 0.000000 [Degree], 0 = horizontal, 90 = vertical
    - R [`floating point number`] : Radius. Minimum: 1.000000 Default: 20.000000 [Cells]
    - D [`floating point number`] : Stripes Distance. Minimum: 2.000000 Default: 2.000000 [Cells]

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '5', 'Destriping')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT3', RESULT3)
        Tool.Set_Output('RESULT1', RESULT1)
        Tool.Set_Output('RESULT2', RESULT2)
        Tool.Set_Output('STRIPES', STRIPES)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R', R)
        Tool.Set_Option('D', D)
        return Tool.Execute(Verbose)
    return False

def Destriping_with_Mask(
    INPUT=None,
    MASK=None,
    RESULT3=None,
    RESULT1=None,
    RESULT2=None,
    STRIPES=None,
    ANG=None,
    R=None,
    D=None,
    MIN=None,
    MAX=None,
    MMIN=None,
    MMAX=None, Verbose=2):
    '''
    Destriping with Mask
    ----------
    [contrib_perego.6]\n
    Destriping filter removes straight parallel stripes in raster data. It uses two low-pass filters elongated in the stripes direction; the first one is 1 pixel unit wide while the second one is wide as the striping wavelength. Their difference is the striping error which is removed from the original data to obtain the destriped DEM. This method is equivalent to that proposed ry[1] Oimoen (2000). With destriping 2 you can choose a range of value (min-max) from the input grid and a range of value (Mask min - Mask max) from a mask grid to select the target cells.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - MASK [`input grid`] : Mask. Mask used to select cells for processing
    - RESULT3 [`output grid`] : Destriped Grid
    - RESULT1 [`output grid`] : Low-pass 1. Step 1: low-pass of stripe
    - RESULT2 [`output grid`] : Low-pass 2. Step 2: low-pass between stripe and its surroundings
    - STRIPES [`output grid`] : Stripes. The difference between destriped and original grid
    - ANG [`floating point number`] : Angle. Default: 0.000000 [Degree], 0 = horizontal, 90 = vertical
    - R [`floating point number`] : Radius. Minimum: 1.000000 Default: 20.000000 [Cells]
    - D [`floating point number`] : Stripes Distance. Minimum: 2.000000 Default: 2.000000 [Cells]
    - MIN [`floating point number`] : Minimum. Default: -10.000000
    - MAX [`floating point number`] : Maximum. Default: 10.000000
    - MMIN [`floating point number`] : Mask Minimum. Default: -10000.000000
    - MMAX [`floating point number`] : Mask Maximum. Default: 10000.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '6', 'Destriping with Mask')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT3', RESULT3)
        Tool.Set_Output('RESULT1', RESULT1)
        Tool.Set_Output('RESULT2', RESULT2)
        Tool.Set_Output('STRIPES', STRIPES)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R', R)
        Tool.Set_Option('D', D)
        Tool.Set_Option('MIN', MIN)
        Tool.Set_Option('MAX', MAX)
        Tool.Set_Option('MMIN', MMIN)
        Tool.Set_Option('MMAX', MMAX)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_6(
    INPUT=None,
    MASK=None,
    RESULT3=None,
    RESULT1=None,
    RESULT2=None,
    STRIPES=None,
    ANG=None,
    R=None,
    D=None,
    MIN=None,
    MAX=None,
    MMIN=None,
    MMAX=None, Verbose=2):
    '''
    Destriping with Mask
    ----------
    [contrib_perego.6]\n
    Destriping filter removes straight parallel stripes in raster data. It uses two low-pass filters elongated in the stripes direction; the first one is 1 pixel unit wide while the second one is wide as the striping wavelength. Their difference is the striping error which is removed from the original data to obtain the destriped DEM. This method is equivalent to that proposed ry[1] Oimoen (2000). With destriping 2 you can choose a range of value (min-max) from the input grid and a range of value (Mask min - Mask max) from a mask grid to select the target cells.\n
    Arguments
    ----------
    - INPUT [`input grid`] : Grid
    - MASK [`input grid`] : Mask. Mask used to select cells for processing
    - RESULT3 [`output grid`] : Destriped Grid
    - RESULT1 [`output grid`] : Low-pass 1. Step 1: low-pass of stripe
    - RESULT2 [`output grid`] : Low-pass 2. Step 2: low-pass between stripe and its surroundings
    - STRIPES [`output grid`] : Stripes. The difference between destriped and original grid
    - ANG [`floating point number`] : Angle. Default: 0.000000 [Degree], 0 = horizontal, 90 = vertical
    - R [`floating point number`] : Radius. Minimum: 1.000000 Default: 20.000000 [Cells]
    - D [`floating point number`] : Stripes Distance. Minimum: 2.000000 Default: 2.000000 [Cells]
    - MIN [`floating point number`] : Minimum. Default: -10.000000
    - MAX [`floating point number`] : Maximum. Default: 10.000000
    - MMIN [`floating point number`] : Mask Minimum. Default: -10000.000000
    - MMAX [`floating point number`] : Mask Maximum. Default: 10000.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '6', 'Destriping with Mask')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Input ('MASK', MASK)
        Tool.Set_Output('RESULT3', RESULT3)
        Tool.Set_Output('RESULT1', RESULT1)
        Tool.Set_Output('RESULT2', RESULT2)
        Tool.Set_Output('STRIPES', STRIPES)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R', R)
        Tool.Set_Option('D', D)
        Tool.Set_Option('MIN', MIN)
        Tool.Set_Option('MAX', MAX)
        Tool.Set_Option('MMIN', MMIN)
        Tool.Set_Option('MMAX', MMAX)
        return Tool.Execute(Verbose)
    return False

def Directional_Average(
    INPUT=None,
    RESULT=None,
    ANG=None,
    R1=None,
    R2=None, Verbose=2):
    '''
    Directional Average
    ----------
    [contrib_perego.7]\n
    directional1 average for Grids\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : Output Grid. New grid filtered with the directional1 tool
    - ANG [`floating point number`] : Angle (in degrees). Default: 0.000000 0 is horizontal, 90 is vertical.
    - R1 [`floating point number`] : Main Radius. Default: 1.000000
    - R2 [`floating point number`] : Transversal radius. Default: 0.500000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '7', 'Directional Average')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R1', R1)
        Tool.Set_Option('R2', R2)
        return Tool.Execute(Verbose)
    return False

def run_tool_contrib_perego_7(
    INPUT=None,
    RESULT=None,
    ANG=None,
    R1=None,
    R2=None, Verbose=2):
    '''
    Directional Average
    ----------
    [contrib_perego.7]\n
    directional1 average for Grids\n
    Arguments
    ----------
    - INPUT [`input grid`] : Input. This must be your input data of type grid.
    - RESULT [`output grid`] : Output Grid. New grid filtered with the directional1 tool
    - ANG [`floating point number`] : Angle (in degrees). Default: 0.000000 0 is horizontal, 90 is vertical.
    - R1 [`floating point number`] : Main Radius. Default: 1.000000
    - R2 [`floating point number`] : Transversal radius. Default: 0.500000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('contrib_perego', '7', 'Directional Average')
    if Tool.is_Okay():
        Tool.Set_Input ('INPUT', INPUT)
        Tool.Set_Output('RESULT', RESULT)
        Tool.Set_Option('ANG', ANG)
        Tool.Set_Option('R1', R1)
        Tool.Set_Option('R2', R2)
        return Tool.Execute(Verbose)
    return False

